@@ -65,6 +65,9 @@ BayonneBinder("ivrscript1")
 			(Check)&Checks::chkJoin},
                 {NULL, false, NULL, NULL}}; 
 
+	char buf[256];
+	const char *cp;
+
 	bind(interp);
 	slog.info("binding ivrscript...");
 	PersistProperty::load();
@@ -73,6 +76,16 @@ BayonneBinder("ivrscript1")
 	Bayonne::use_prefix = true;
 	Bayonne::use_macros = true;
 	Bayonne::use_merge = false;
+
+	cp = server->getLast("serverconfig");
+	if(cp)
+	{
+		snprintf(buf, sizeof(buf), "%s/%s/connect.conf",
+			CONFIG_FILES, cp);
+		server->setValue("connect.conf", buf);
+	}
+	else
+		server->setValue("connect.conf", CONFIG_FILES "/connect.conf");
 };
 
 bool Binder::select(ScriptInterp *interp)
@@ -224,6 +237,7 @@ bool Binder::reload(ScriptCompiler *img)
 
 #endif
 
+	
 	count = server->Keydata::getIndex(keys, count);
 	while(count)
 	{
@@ -240,7 +254,6 @@ bool Binder::reload(ScriptCompiler *img)
 
 		if(!ext || stricmp(ext, ".scr"))
 			continue;
-
 compile:
 		++Bayonne::compile_count;
 		prefix = server->getLast(prefix);
