@@ -60,6 +60,7 @@ bool Methods::scrStart(void)
 	char buf[65];
 	TelTone::tonekey_t *key = NULL;
 	Audio::Level level = 26000;
+	bool connect_flag = false;
 
 	if(!stricmp(line->cmd, "connect"))
 	{
@@ -67,21 +68,30 @@ bool Methods::scrStart(void)
 		state.timeout = getTimeoutKeyword("timeout");
 		state.tone.duration = getTimeoutKeyword("duration");
 		state.tone.dtmf = true;
-		if(state.timeout && state.timeout != TIMEOUT_INF)
+
+		if(!caller)
+			caller = getSymbol("session.caller");
+
+		if(!display)
+			display = getSymbol("session.display");
+
+		if(audio.tone)
 		{
-			if(audio.tone)
-			{
-				delete audio.tone;
-				audio.tone = NULL;
-			}
-			cp = getKeyword("tone");
-			if(cp && !stricmp(cp, "ringback"))
-				cp = "ring";
-			if(cp)
-				key = TelTone::find(cp, Bayonne::server->getLast("location"));
-			if(key)
-				audio.tone = new TelTone(key, level, getToneFraming());
+			delete audio.tone;
+			audio.tone = NULL;
 		}
+		cp = getKeyword("tone");
+		if(!cp)
+			cp = "ring";
+		if(cp && !stricmp(cp, "ringback"))
+			cp = "ring";
+		if(cp && stricmp(cp, "none"))
+			key = TelTone::find(cp, Bayonne::server->getLast("location"));
+		if(key)
+			audio.tone = new TelTone(key, level, getToneFraming());
+		else if(stricmp(cp, "none"))
+			slog.debug("%s: connect; cannot find tone %s", logname, cp);
+		connect_flag = true;
 	}
 	
 	if(!dial || !name)
@@ -89,7 +99,7 @@ bool Methods::scrStart(void)
 		if(sym)
 			ScriptInterp::commit(sym, "none");
 		if(!scriptEvent("start:invalid"))
-			error("start-invalid");
+			error("start-incomplete");
 		return true;
 	}
 
@@ -123,6 +133,9 @@ bool Methods::scrStart(void)
 		child->setConst(id, sym->data);
 	}
 
+	if(connect_flag)
+		child->startConnecting();
+
 	child->leave();
 	if(!stricmp(line->cmd, "connect"))
 	{
