@@ -38,6 +38,7 @@ timeout_t Bayonne::exec_timer = 60000;
 Bayonne::Traffic Bayonne::total_call_attempts;
 Bayonne::Traffic Bayonne::total_call_complete;
 Mutex Bayonne::serialize;
+ThreadLock Bayonne::reloading;
 char Bayonne::sla[64] = {0};
 unsigned Bayonne::idle_count = 0;
 unsigned Bayonne::idle_limit = 0;
@@ -539,8 +540,6 @@ void Bayonne::down(void)
 
 ScriptCompiler *Bayonne::reload(void)
 {
-	static Mutex lock;
-
 	ScriptCompiler *img;
 
 	if(!server)
@@ -548,7 +547,7 @@ ScriptCompiler *Bayonne::reload(void)
 
 	compile_count = 0;
 
-	lock.enter();
+	reloading.writeLock();
 	img = new ScriptCompiler(server, "/bayonne/server/config");
 	if(getUserdata())
 		img->loadPrefix("config", "~bayonne/config");
@@ -562,7 +561,7 @@ ScriptCompiler *Bayonne::reload(void)
 	BayonneConfig::rebuild(img);	
 	ScriptBinder::rebuild(img);
 	img->commit();
-	lock.leave();
+	reloading.unlock();
 	if(compile_count)
 		image_loaded = true;
 	return img;
@@ -828,10 +827,17 @@ protocol:
 		sel = scr->select;
 
 	cp = d->getLast("urlprefix");
+	if(sel && *sel->args[0] == '-')
+	{
+		sel = NULL;
+		cp = "";
+	}
+
 	if(sel)
 		snprintf(buffer, sizeof(buffer), "%s%s@%s", cp, dial, sel->args[0]); 
 	else
 		snprintf(buffer, sizeof(buffer), "%s%s", cp, dial);
+
 	dial = buffer;
 	goto start;
 
