@@ -151,6 +151,7 @@ public:
 		FORWARDED,	/* inbound forwarded call */
 		RECALL,		/* inbound hold recall */
 		DIRECT,		/* node/endpoint direct */
+		RINGING,	/* ringing incoming join */
 		VIRTUAL		/* virtual channel driver */
 	}	calltype_t;
 
@@ -713,6 +714,11 @@ public:
 	 */
 	static Mutex serialize;
 
+	/**
+	 * A mutex to serialize reload requests.
+	 */
+	static ThreadLock reloading;
+
 	/**
 	 * master traffic counters for call attempts and call completions.
 	 */
@@ -1904,7 +1910,7 @@ protected:
 	uint8 seq;
 	uint16 evseq;
 	uint32 tseq;
-	bool offhook, dtmf, answered, starting, holding;
+	bool offhook, dtmf, answered, starting, holding, connecting;
 	time_t audiotimer, starttime;
 	interface_t iface;
 	bridge_t bridge;
@@ -2132,6 +2138,12 @@ protected:
 	 */
 	void setRunning(void);
 
+	/**
+	 * Set the session to the connecting (join) state or run state
+	 * based on flags and circumstances from seize/pickup.
+	 */
+	void setConnecting(const char *evname = NULL);
+
 	/**
 	 * Set the result of a libexec initiated process and change
 	 * state to libexec.  Return false if no libexec support, or
@@ -2161,6 +2173,12 @@ public:
 	inline BayonneTranslator *getTranslator(void)
 		{return translator;};
 
+	/**
+	 * Start connecting child...
+	 */
+	inline void startConnecting(void)
+		{connecting = true;};
+
 protected:
 	/**
 	 * Get the libaudio object.  Used by server libexec to prepare
