@@ -49,6 +49,7 @@ ScriptInterp(), audio()
 	translator = init_translator;
 	state.menu = NULL;
 	peer = NULL;
+	connecting = false;
 
 	if(!translator)
 		translator = &langNone;
@@ -471,6 +472,8 @@ const char *BayonneSession::getExternal(const char *id)
 				return "pickup";
 			case VIRTUAL:
 				return "virtual";
+			case RINGING:
+				return "ringing";
 			}
 		}
 		else if(!stricmp(id, "interface"))
@@ -906,6 +909,43 @@ bool BayonneSession::setLibexec(result_t result)
 	return true;
 }
 
+void BayonneSession::setConnecting(const char *evname)
+{
+	char scrbuf[65];
+	Name *scr;
+
+	if(evname)
+	{
+		if(scriptEvent(evname))
+		{
+			setRunning();
+			return;
+		}
+		snprintf(scrbuf, sizeof(scrbuf), "connect::%s", evname + 5);
+		scr = getScript(scrbuf);
+		if(connecting && scr)
+		{
+			redirect(scrbuf);
+			return;
+		}
+	}
+
+	switch(type)
+	{
+	case RINGING:
+		if(!scriptEvent("call:incoming") && connecting)
+			redirect("connect::incoming");
+		break;
+	case OUTGOING:
+		if(connecting && !scriptEvent("call:outgoing"))
+			redirect("connect::outgoing");
+		break;
+	default:
+		break;
+	}
+	setRunning();
+}
+
 void BayonneSession::setRunning(void)
 {
 	setState(STATE_RUNNING);
@@ -1296,6 +1336,7 @@ void BayonneSession::makeIdle(void)
         strcpy(var_rings, "0");
         var_sid[0] = 0;
 	holding = false;
+	connecting = false;
         sym = mapSymbol("session.digits", MAX_DTMF);
         if(sym)
 	{
