@@ -1561,22 +1561,42 @@ bool BayonneSession::statePickup(Event *event)
 	{
 	case ENTER_STATE:
 		if(image && offhook)
-		{
-			setRunning();
-			return true;
-		}
+			goto running;
 		setOffhook(true);
 		startTimer(driver->getPickupTimer());
 		return true;
 	case LINE_OFF_HOOK:
 	case TIMER_EXPIRED:
-		setRunning();
-		return true;
+		goto running;
 	case LINE_DISCONNECT:
 		return true;
 	default:
 		return enterCommon(event);
 	}
+
+running:
+	switch(type)
+	{
+	case INCOMING:
+		break;
+	case OUTGOING:
+	case RINGING:
+		setConnecting();
+		return true;
+	case PICKUP:
+		scriptEvent("call:pickup");
+		break;
+	case FORWARDED:
+		scriptEvent("call:forward");
+		break;
+	case RECALL:
+		scriptEvent("call:recall");
+		break;
+	default:
+		break;
+	}
+	setRunning();
+	return true;
 }		
 
 bool BayonneSession::stateSeize(Event *event)
@@ -1596,20 +1616,20 @@ bool BayonneSession::stateSeize(Event *event)
 
 		if(image && offhook)
 		{
-			setRunning();
+			setConnecting();
 			return true;
 		}
 		setOffhook(true);
 		if(driver->getSeizeTimer())
 			startTimer(driver->getSeizeTimer());
 		else
-			setRunning();
+			setConnecting();
 		return true;
 	case TONE_START:
 		if(state.join.dial)
 			return true;
 
-		snprintf(evtname, sizeof(evtname), "seize:%s", event->tone.name);
+		snprintf(evtname, sizeof(evtname), "call:%s", event->tone.name);
   		if(scriptEvent(evtname))
 		{
 			setRunning();
@@ -1619,8 +1639,13 @@ bool BayonneSession::stateSeize(Event *event)
 		{
 			state.join.dial = getSymbol("session.dialed");
 			if(state.join.dial)
-				goto dialer;
-			setRunning();
+			{
+				event->id = ENTER_STATE;
+				if(enterDial(event))
+					return true;
+				goto failed;
+			}
+			setConnecting();
 			return true;
 		}
 		if(event->tone.exit)
@@ -1631,17 +1656,15 @@ bool BayonneSession::stateSeize(Event *event)
 		return true;
 	case DIAL_FAX:
 		seizure = CHILD_FAX;
-		scriptEvent("seize:fax");
-		setRunning();
+		setConnecting("call:faxout");
 		return true;
 	case DIAL_PAM:
 		seizure = CHILD_RUNNING;
-		scriptEvent("seize:pam");
-		setRunning();
+		setConnecting("call:machine");
 		return true;
 	case DIAL_INVALID:
 		seizure = CHILD_INVALID;
-		if(scriptEvent("seize:invalid"))
+		if(scriptEvent("call:invalid"))
 		{
 			setRunning();
 			return true;
@@ -1650,7 +1673,7 @@ bool BayonneSession::stateSeize(Event *event)
 		return true;
 	case DIAL_BUSY:
 		seizure = CHILD_BUSY;
-		if(scriptEvent("seize:busy"))
+		if(scriptEvent("call:busy"))
 		{
 			setRunning();
 			return true;
@@ -1658,15 +1681,13 @@ bool BayonneSession::stateSeize(Event *event)
 		setState(STATE_HANGUP);
 		return true;
 	case DIAL_CONNECT:
-		snprintf(evtname, sizeof(evtname), "dialed:%s", state.join.dial);
-		digitEvent(evtname);	
 		seizure = CHILD_RUNNING;
-		setRunning();
+		setConnecting();
 		return true;
 	case DIAL_TIMEOUT:
 noanswer:
 		seizure = CHILD_EXPIRED;
-		if(scriptEvent("seize:noanswer"))
+		if(scriptEvent("call:noanswer"))
 		{
 			setRunning();
 			return true;
@@ -1680,7 +1701,7 @@ noanswer:
 	case DIAL_FAILED:
 failed:
 		seizure = CHILD_FAILED;
-		if(scriptEvent("seize:failed"))
+		if(scriptEvent("call:failed"))
 		{
 			setRunning();
 			return true;
@@ -1693,27 +1714,6 @@ failed:
 	default:
 		return enterCommon(event);
 	}
-
-dialer:
-	snprintf(evtname, sizeof(evtname), "reject:%s", state.join.dial);
-	if(digitEvent(evtname))
-	{
-		seizure = CHILD_INVALID;
-		setRunning();
-		return true;
-	}
-	snprintf(evtname, sizeof(evtname), "accept:%s", state.join.dial);
-	if(digitEvent(evtname))
-	{
-		seizure = CHILD_RUNNING;
-		setRunning();
-		return true;
-	}
-	event->id = ENTER_STATE;
-	if(enterDial(event))
-		return true;
-
-	goto failed;
 }		
 
 bool BayonneSession::stateRunning(Event *event)
@@ -1737,6 +1737,7 @@ bool BayonneSession::stateRunning(Event *event)
 			setConst("session.callref", var_sid);
 			switch(type)
 			{
+			case RINGING:
 			case OUTGOING:
 				incOutgoingComplete();
 			case VIRTUAL:
