@@ -656,30 +656,9 @@ BayonneTranslator *BayonneTranslator::loadTranslator(const char *id)
 	const char *prefix = NULL;
 
 #ifdef  WIN32
-#ifdef	HAVE_TESTING
-        if(!prefix && provision_test)
-#ifdef  NDEBUG
-                prefix = "DEBUG";
-#else    
-                prefix = "RELEASE";
-#endif 
-#endif
         if(!prefix)
                 prefix="C:\\Program Files\\Common Files\\GNU Telephony\\Bayonne Translators";
 #else
-#ifdef	HAVE_TESTING
-        char prefixbuf[256];
-
-        if(!prefix && provision_test)
-        {
-		snprintf(prefixbuf, sizeof(prefixbuf), 
-			"%s/modules/languages/.libs", SOURCE_FILES);
-                if(!isDir(prefixbuf))
-			snprintf(prefixbuf, sizeof(prefixbuf), 
-				"%s/modules/languages", SOURCE_FILES);
-                prefix = prefixbuf;
-        } 
-#endif
         if(!prefix)
                 prefix=LIBDIR_FILES;
 #endif  
