@@ -53,6 +53,7 @@ static Keydata::Define driver[] = {
 	{"immediate", "yes"},
 	{"peering", "friend"},	// default registration type
 	{"starting", "delayed"},
+	{"threads", "5"},
 	{"realm", "Bayonne"},
 	{"nonce", "456345766asd345"},
 #ifdef	WIN32		// use localhost default if not in conf for w32
@@ -76,7 +77,9 @@ static char *remove_quotes(char *c)
 	return o;
 }
 
+SIPThread *SIPThread::first = NULL;
 Driver Driver::sip;
+unsigned Driver::instance = 0;
 
 bool Registry::isActive(void)
 {
@@ -102,6 +105,18 @@ void Registry::add(ScriptImage *img)
         img->setPointer("_sip_registry_", this);
 }
 
+SIPThread::SIPThread() :
+Thread(atoi(Driver::sip.getLast("priority")), atoi(Driver::sip.getLast("stack")) * 1024 + 8192)
+{
+	next = first;
+	first = this;
+}
+
+void SIPThread::run(void)
+{
+	Driver::sip.run();
+}
+
 Driver::Driver() :
 BayonneDriver(driver, "/bayonne/driver/sip", "sip", false),
 Thread(atoi(getLast("priority")), atoi(getLast("stack")) * 1024 + 8192)
@@ -308,7 +323,7 @@ unsigned Driver::getRegistration(regauth_t *data, unsigned count)
 	return idx;
 }
 
-bool Driver::getAuthentication(Session *s)
+bool Driver::getAuthentication(Session *s, osip_message_t *resp)
 {
 	char buf[256];
 	ScriptImage *img = s->getImage();
@@ -316,14 +331,30 @@ bool Driver::getAuthentication(Session *s)
 	char *p;
 	Registry *reg;
 	bool rtn = false;
+	const char *realm;
+        osip_proxy_authenticate_t *prx_auth = NULL;
+        osip_www_authenticate_t *www_auth = NULL;
+
+        if(resp)
+        {
+		prx_auth = (osip_proxy_authenticate_t*)osip_list_get(resp->proxy_authenticates, 0);
+                www_auth = (osip_proxy_authenticate_t*)osip_list_get(resp->www_authenticates,0);
+        }
 
 	if(!cp)
 		goto done;
 
-	reg = (Registry *)img->getPointer(buf);
+	reg = (Registry *)img->getPointer(cp);
 	if(!reg)
 		goto done;
 
+        if(prx_auth)
+                realm = osip_proxy_authenticate_get_realm(prx_auth);
+        else if(www_auth)
+                realm = osip_www_authenticate_get_realm(www_auth);
+	else
+		realm = reg->realm;
+
 	if(!reg->userid)
 		goto done;
 
@@ -333,7 +364,7 @@ bool Driver::getAuthentication(Session *s)
 		*(p++) = 0;
 
 	eXosip_lock();
-	if(eXosip_add_authentication_info(buf, buf, reg->secret, NULL, reg->realm))
+	if(eXosip_add_authentication_info(buf, buf, reg->secret, NULL, realm))
 		slog.error("sip: authentication failed; host=%s", reg->proxy);
 	else
 	{
@@ -345,18 +376,32 @@ done:
 	return rtn;
 }
 
-void Driver::setAuthentication(Registry *reg)
+void Driver::setAuthentication(Registry *reg, osip_message_t *resp)
 {
+        osip_proxy_authenticate_t *prx_auth = NULL;
+        osip_www_authenticate_t *www_auth = NULL;
 	char uname[65];
+	const char *realm = reg->realm;
 	char *p;
 
+	if(resp)
+	{
+		prx_auth = (osip_proxy_authenticate_t*)osip_list_get(resp->proxy_authenticates, 0);
+        	www_auth = (osip_proxy_authenticate_t*)osip_list_get(resp->www_authenticates,0);
+	}
+
+	if(prx_auth)
+		realm = osip_proxy_authenticate_get_realm(prx_auth);
+	else if(www_auth)
+		realm = osip_www_authenticate_get_realm(www_auth);
+
 	setString(uname, sizeof(uname), reg->userid);
 	p = strchr(uname, '@');
 	if(p)
 		*p = 0;
 
 	eXosip_lock();
-	eXosip_add_authentication_info(uname, uname, reg->secret, NULL, reg->realm);
+	eXosip_add_authentication_info(uname, uname, reg->secret, NULL, realm);
 	eXosip_unlock();
 }
 		
@@ -373,6 +418,8 @@ void Driver::startDriver(void)
 	exiting = false;
 	timeslot = ts_used;
 	msgport = new BayonneMsgport(this);
+	unsigned threads = 0;
+	SIPThread *thread;
 	
 	if(cp)
 		max_count = atoi(cp);
@@ -413,6 +460,8 @@ void Driver::startDriver(void)
 			rtp_port += 10;
 	}
 
+	threads = atoi(getLast("threads"));
+
 	cp = getLast("interface");
 	if(!cp)
 		cp = "all";
@@ -497,6 +546,13 @@ void Driver::startDriver(void)
 	slog.debug("sip: bound to port %d", sip_port);
 	msgport->start();
 	Thread::start();
+
+	while(threads-- > 1)
+	{
+		thread = new SIPThread();
+		thread->start();
+	}
+
 	Thread::sleep(100);
 	BayonneDriver::startDriver();	
 
@@ -511,11 +567,34 @@ void Driver::startDriver(void)
 
 void Driver::stopDriver(void)
 {
+	ScriptImage *img;
+	Registry *reg;
+	SIPThread *thread;
+
 	if(running)
 	{
 		exiting = true;
+		img = useImage();
+		if(img)
+		{
+			reg = (Registry *)img->getPointer("_sip_registry_");
+			while(reg)
+			{
+				if(strnicmp(reg->type, "ext", 3))
+					updateExpires(reg, 0);
+				reg = reg->next;
+			}
+			endImage(img);
+		}
 		__eXosip_wakeup_event();
 		terminate();
+		thread = SIPThread::first;
+		while(thread)
+		{
+			thread->terminate();
+			thread = thread->next;
+		}
+		SIPThread::first = NULL;
 		eXosip_quit();
 		BayonneDriver::stopDriver();
 	}
@@ -686,6 +765,18 @@ bool Driver::getRegistry(eXosip_event_t *sevent, Registry **reg, ScriptImage *im
 	return false;
 }
 
+void Driver::updateExpires(Registry *reg, timeout_t expires)
+{
+	osip_message_t *msg = NULL;
+
+	reg->duration = expires;
+	eXosip_lock();
+	eXosip_register_build_register(reg->regid, (int)(expires / 1000), &msg);
+	if(msg)
+		eXosip_register_send_register(reg->regid, msg);
+	eXosip_unlock();
+}
+
 void Driver::registerRequest(eXosip_event_t *sevent)
 {
 	char a1[256], a2[256], a1_hash[256], a2_hash[256], resp[256], resp_hash[256];
@@ -828,6 +919,8 @@ const char *Driver::registerScript(ScriptImage *img, Line *line)
 	osip_message_t *msg = NULL; 
 	InetAddress host;
 	bool ext = false;
+	Registry *preg;
+	ScriptImage *pimg;
 
 	memset(reg, 0, sizeof(Registry));
 
@@ -837,13 +930,12 @@ const char *Driver::registerScript(ScriptImage *img, Line *line)
 	if(ext)
 	{
 		if(!ScriptChecks::useKeywords(line, 
-			"=secret=encoding=framing=dtmf=forward"))
+			"=secret=encoding=framing=dtmf=forward=user"))
 				return "invalid keywords used for sip external";
 	}
-
-	if(!ScriptChecks::useKeywords(line, 
+	else if(!ScriptChecks::useKeywords(line, 
 		"=uri=timeout=server=proxy=userid=secret=type=realm=dtmf"
-		"=limit=public=encoding=framing=forward"))
+		"=limit=public=encoding=framing=local=caller"))
 			return "invalid keywords for sip registry";
 
 	reg->updated = 0l;
@@ -860,6 +952,12 @@ const char *Driver::registerScript(ScriptImage *img, Line *line)
 	reg->dtmf = ScriptChecks::findKeyword(line, "dtmf");
 	reg->encoding = ScriptChecks::findKeyword(line, "encoding");
 	reg->forward = ScriptChecks::findKeyword(line, "forward");
+	reg->caller = ScriptChecks::findKeyword(line, "caller");
+
+	if(ext)
+		reg->localid = ScriptChecks::findKeyword(line, "user");
+	else
+		reg->localid = ScriptChecks::findKeyword(line, "local");
 
 	cp = ScriptChecks::findKeyword(line, "limit");
 	if(cp)
@@ -963,8 +1061,11 @@ const char *Driver::registerScript(ScriptImage *img, Line *line)
 	}
 	
 	len = strlen(buf) - 3;
-	reg->localid = (const char *)img->getMemory(len);
-	strcpy((char *)reg->localid, buf + 4);
+	if(!reg->localid)
+	{
+		reg->localid = (const char *)img->getMemory(len);
+		strcpy((char *)reg->localid, buf + 4);
+	}
         if(!reg->userid)
                 reg->userid = reg->localid; 
 
@@ -1009,13 +1110,44 @@ const char *Driver::registerScript(ScriptImage *img, Line *line)
 	{
 		reg->type = "anon";
 		slog.debug("register %s for anonymous calling", reg->localid);
-		img->setValue("sip.anonymous", reg->localid);
+		if(reg->address)
+		{
+			snprintf(buf, sizeof(buf), "sipanon.%s", reg->address);
+			img->setPointer(buf, reg);
+		}
+		else
+			img->setPointer("sipanon.*", reg);
 		goto add;
 	}
 
-	if(!stricmp(reg->type, "node") || !strnicmp(reg->type, "ext", 3))
-	{
+	if(ext)
 		reg->type = "extern";
+
+	pimg = useImage();
+	preg = NULL;
+	if(pimg)
+	{
+		snprintf(buf, sizeof(buf), "uri.%s", reg->localid);
+		preg = (Registry *)pimg->getPointer(buf);
+	}
+			
+	if(preg && preg->active && !stricmp(reg->type, preg->type))
+	{
+		reg->active = true;
+		reg->setTimer(preg->getTimer());
+		if(ext && preg->proxy)
+		{
+			reg->proxy = (const char *)img->getMemory(strlen(preg->proxy) + 1);
+			strcpy((char *)reg->proxy, preg->proxy);
+			reg->updated = preg->updated;
+		}
+	}
+
+	if(pimg)
+		endImage(pimg);
+
+	if(ext)
+	{
 		slog.debug("register %s externally", reg->localid);
 		goto add;
 	}
@@ -1033,7 +1165,7 @@ const char *Driver::registerScript(ScriptImage *img, Line *line)
 		return "cannot create sip registration";
 
 	if(reg->secret)
-		setAuthentication(reg);
+		setAuthentication(reg, NULL);
 
 //	osip_message_set_supported(msg, "path");   
 	osip_message_set_header(msg, "Event", "Registration");
@@ -1130,6 +1262,7 @@ void Driver::run(void)
 	char digit[10];
 	char duration[10];
 	osip_body_t *mbody;
+	osip_header_t *header;
 	const char *auth = "anon";
 	timeout_t event_timer = atoi(getLast("timer"));
 	osip_message_t *msg = NULL;
@@ -1146,6 +1279,8 @@ void Driver::run(void)
 	char *sdp;
 	bool reg_inband;
 	uint8 reg_payload;
+	timeout_t expires;
+	unsigned thread = ++instance;
 
 	Bayonne::waitLoaded();
 
@@ -1164,8 +1299,8 @@ void Driver::run(void)
 		if(!sevent)
 			continue;
 
-		slog.debug("sip: event %04x; cid=%d, did=%d, rid=%d",
-			sevent->type, sevent->cid, sevent->did, sevent->rid);
+		slog.debug("sip: event %04x; cid=%d, did=%d, rid=%d, instance=%d",
+			sevent->type, sevent->cid, sevent->did, sevent->rid, thread);
 
 		switch(sevent->type)
 		{
@@ -1177,23 +1312,47 @@ void Driver::run(void)
 			server->enter();
 			img = server->getActive();
 			reg = (Registry *)img->getPointer(buf);	
-			if(reg)
+			header = NULL;
+			osip_message_get_expires(sevent->request, 0, &header);
+			expires = 0;
+			if(header)
+				expires = osip_atoi(header->hvalue) * 1000l;
+			else if(reg)
+				expires = reg->duration;
+			if(reg && !expires)
+			{
+				reg->active = false;
+				slog.debug("registration for %s rejected", reg->contact);
+			}
+			else if(reg)
 			{
 				reg->active = true;
-				reg->setTimer(reg->duration);
-				slog.debug("registration for %s confirmed", reg->contact);
+				reg->setTimer(expires);
+				if(expires != reg->duration)
+				{
+					updateExpires(reg, expires);
+					slog.debug("registration for %s updated for %d seconds",
+						reg->contact, reg->duration / 1000);			
+				}
+				else
+					slog.debug("registration for %s confirmed for %d seconds", 
+						reg->contact, expires / 1000);
 			}
 			else
 				slog.warn("unknown sip registration confirmed; rid=%s", sevent->rid);
 			server->leave();
 			break;
 		case EXOSIP_REGISTRATION_FAILURE:
-			if(sevent->response && sevent->response->status_code == 401)
-				break;				
 			snprintf(buf, sizeof(buf), "sipreg.%d", sevent->rid);
 			server->enter();
 			img = server->getActive();
 			reg = (Registry *)img->getPointer(buf);	
+			if(reg && sevent->response && sevent->response->status_code == 401)
+			{
+				setAuthentication(reg, sevent->response);
+				server->leave();
+				break;
+			}
 			if(reg)
 			{
 				reg->setTimer(0);
@@ -1326,6 +1485,7 @@ void Driver::run(void)
 			}
 exitcall:
 			memset(&event, 0, sizeof(event));
+			slog.debug("%s: closing call", session->logname);
 			event.id = STOP_DISCONNECT;	
 			session->did = 0;
 			session->postEvent(&event);
@@ -1335,6 +1495,7 @@ exitcall:
 			if(!session)
 				break;
 			memset(&event, 0, sizeof(event));
+			session->did = sevent->did;
 			event.id = CALL_PROCEEDING;
 			session->postEvent(&event);
 			break;
@@ -1343,6 +1504,7 @@ exitcall:
                         if(!session)
                                 break;
                         memset(&event, 0, sizeof(event));
+			session->did = sevent->did;
                         event.id = CALL_RINGING;
                         session->postEvent(&event);
                         break;
@@ -1350,6 +1512,7 @@ exitcall:
                         session = getCall(sevent->cid);
                         if(!session)
                                 break;
+			session->did = sevent->did;
 			msg = NULL;
 			sdp = NULL;
 			local_sdp = NULL;
@@ -1466,6 +1629,7 @@ killit:
 				eXosip_unlock();
 				break;
 			}
+			session->did = sevent->did;
 			memset(&event, 0, sizeof(event));
 			event.id = CALL_ACCEPTED;
 			session->postEvent(&event);
@@ -1478,16 +1642,20 @@ killit:
 				break;
 
 			memset(&event, 0, sizeof(event));
+			slog.debug("sip: failure result code %d", sevent->response->status_code);
 			switch(sevent->response->status_code)
 			{
 			case AUTH_REQUIRED:
-				getAuthentication(session);
+				getAuthentication(session, sevent->response);
 				eXosip_lock();
 				eXosip_automatic_action();
 				eXosip_unlock();
 				break;
 			case UNAUTHORIZED:
-				getAuthentication(session);
+				getAuthentication(session, sevent->response);
+				eXosip_lock();
+				eXosip_automatic_action();
+				eXosip_unlock();
 				break;
 			case BUSY_HERE:
 				event.id = DIAL_BUSY;
@@ -1574,7 +1742,8 @@ killit:
 
 			img = useImage();
 
-                        if(to->url->port && stricmp(to->url->port, "5060"))                                     snprintf(buf, sizeof(buf), "sip.%s:%s",
+                        if(to->url->port && stricmp(to->url->port, "5060"))
+                                     snprintf(buf, sizeof(buf), "sip.%s:%s",
                                         to->url->host, to->url->port);
                         else
                                 snprintf(buf, sizeof(buf), "sip.%s",
@@ -1647,16 +1816,13 @@ killit:
 
 			if(!reg && !peer)
 			{
-				cp = img->getLast("sip.anonymous");
-				if(cp)
-				{
-					snprintf(buf, sizeof(buf), 
-						"uri.%s", cp);
-					reg = (Registry *)img->getPointer(buf);
-					if(reg && !stricmp(reg->type, "anon"))
-						goto validate;
-					reg = NULL;
-				}								
+				reg = (Registry *)img->getPointer("sipanon.*");
+				if(reg)
+					goto validate;
+				snprintf(buf, sizeof(buf), "sipanon.%s", to->url->host);
+				reg = (Registry *)img->getPointer(buf);
+				if(reg)
+					goto validate;
 				endImage(img);
 				requestAuth(sevent);
 				goto clear;
@@ -1860,7 +2026,7 @@ sip415:
 				break;
 			case g721ADPCM:
 				session->data_payload = 5;
-				data_rtpmap = "G726_32/8000/1";
+				data_rtpmap = "G726-32/8000/1";
 				break;
 			case speexVoice:
 				session->data_payload = 97;
@@ -1934,7 +2100,7 @@ sip415:
 						continue;
 					}
 					if(!strnicmp(tmp, "G721", 4))
-						tmp = "G726_32";
+						tmp = "G726-32/8000";
 					if(!strnicmp(tmp, data_rtpmap, strlen(tmp)))
 					{
 						audio_payload = adp;
