@@ -88,6 +88,7 @@ public:
 	const char *dtmf;
 	const char *encoding;
 	const char *forward;
+	const char *caller;
 	timeout_t framing;
 	int regid;
 	bool active;
@@ -100,11 +101,25 @@ public:
 	void add(ScriptImage *img);
 };
 	
+class SIPThread : public Thread
+{
+private:
+	friend class Driver;
+
+	void run(void);
+
+	static SIPThread *first;
+	SIPThread *next;
+
+	SIPThread();
+};
+
 class Driver : public BayonneDriver, public Audio, public Thread
 {
 protected:
 	friend class Session;
 	friend class RTPStream;
+	friend class SIPThread;
 
 	InetAddress sip_addr, rtp_addr;
 	tpport_t rtp_port, sip_port;
@@ -121,6 +136,7 @@ protected:
 	timeout_t accept_timer;
 	timeout_t audio_timer;
 	starting_mode_t starting;
+	static unsigned instance;
 
 public:
 	static Driver sip;		// plugin activation
@@ -138,11 +154,12 @@ public:
 	void registerRequest(eXosip_event_t *sevent);
 	const char *registerScript(ScriptImage *img, Line *line);
 	const char *assignScript(ScriptImage *img, Line *line);
+	void updateExpires(Registry *reg, timeout_t expires);
 
 	Session *getCall(int callid);
 
-	void setAuthentication(Registry *reg);
-	bool getAuthentication(Session *s);
+	void setAuthentication(Registry *reg, osip_message_t *response);
+	bool getAuthentication(Session *s, osip_message_t *response);
 
 	void run(void);
 
