@@ -276,6 +276,7 @@ bool Session::enterSeize(Event *event)
 	const char *route;
         const char *lp = driver->getLast("localip");
 	BayonneSession *parent = Bayonne::getSid(var_pid);
+	const char *caller = NULL;
 
 	switch(event->id)
 	{
@@ -373,6 +374,8 @@ dialing:
 			setConst("session.registry", rbuf);
 			setConst("session.uri_server", reg->proxy);
 			setConst("session.uri_local", reg->contact);
+			setConst("session.uri_caller", reg->uri);
+			caller = reg->caller;
                         if(reg->address)
                         {
                                 InetAddress host(reg->address);
@@ -388,12 +391,23 @@ dialing:
 			snprintf(buf, sizeof(buf), "sip:anon@%s",
 				Driver::sip.getLast("interface"));
 			setConst("session.uri_local", buf);
+			setConst("session.uri_caller", buf);
                         setConst("session.ip_local", lp);
                         setConst("session.ip_public", lp);
 		}
 			
 		to = getSymbol("session.uri_remote");
-		from = getSymbol("session.uri_local");
+		from = getSymbol("session.uri_caller");
+
+		if(!caller)
+		{
+			caller = getSymbol("session.display");
+			if(!caller)
+				caller = getSymbol("session.caller");
+
+			if(caller && !*caller)
+				caller = "unknown";
+		}
 
 		if(!route)
 		{
@@ -405,12 +419,12 @@ dialing:
 		if(!strnicmp(route, "sip:", 4))
 			route += 4;
 
+		snprintf(sdp, sizeof(sdp), "\"%s\" <%s>", caller, from);
 		snprintf(buf, sizeof(buf), "<sip:%s;lr>", route);
-
 		slog.debug("invite %s from %s using %s", to, from, route);
 		eXosip_lock();
 		if(eXosip_call_build_initial_invite(&invite, 
-			(char *)to, (char *)from, buf, "Bayonne Call"))
+			(char *)to, (char *)sdp, buf, "Bayonne Call"))
 		{
 			eXosip_unlock();
 failure:
@@ -418,6 +432,7 @@ failure:
 			event->id = DIAL_FAILED;
 			return false;
 		}
+		osip_message_set_supported(invite, "100rel,replaces");
 		eXosip_unlock();
 		cp = Driver::sip.getLast("localip");
 		data_payload = Driver::sip.data_negotiate;
@@ -519,7 +534,7 @@ failure:
 		return true;
 	case CALL_CONNECTED:
 		startRTP();
-		setRunning();
+		setConnecting();
 		return true;
 	case CALL_ANSWERED:
 		setState(STATE_PICKUP);
@@ -544,7 +559,6 @@ bool Session::enterHangup(Event *event)
 	{
 	case ENTER_STATE:
 		cp = getSymbol("session.registry");
-		printf("SESSION HANGUP REG %s\n", cp);
 		if(!cp)
 			break;
 		reg = (Registry *)img->getPointer(cp);
@@ -692,6 +706,7 @@ void Session::clrAudio(void)
 
 void Session::sipHangup(void)
 {
+	slog.debug("%s: terminating call; cid=%d, did=%d", logname, cid, did);
 	eXosip_lock();
 	eXosip_call_terminate(cid, did);
 	eXosip_unlock();
