@@ -191,6 +191,7 @@ void Session::xmlRegistrations(void)
 			d = d->getNext();
 			continue;
 		}
+		reloading.readLock();
 		count = d->getRegistration(regs, 1024);
 		r = regs;
 		while(count--)
@@ -210,6 +211,7 @@ void Session::xmlRegistrations(void)
 			sendText(buffer);
 			++r;
 		}
+		reloading.unlock();
 		d = d->getNext();
 	}
 	sendText("  </lists>\r\n");
